'''
This function is an example of how to uses pymathis.
The input file is the one made using the MakeFileExample.py script
It's named 'TestCase' and output files will be written in 'TestCase' folder
'''

import os
import pymathis.pymathis as pm
import matplotlib.pyplot as plt

def main():
    # Paths and case name
    C_FILE_Path = os.path.join(os.getcwd(), 'TestCase')
    C_FILE = 'TestCase.data'

    #lets go in the right folder and initialise connection with MATHIS
    os.chdir(C_FILE_Path)
    SmartMathis =pm.LoadDll(pm.__file__,C_FILE)

    # Lets fetch the available actuators and their IDs, initialization value is set to 0
    n_passive = pm.get_passive_number(SmartMathis)
    PassiveCtrl_ID = {pm.get_ctrl_ID(SmartMathis, i):[0] for i in range(n_passive)}
    # lets fetch the probes and their IDs
    n_probe = pm.get_probe_number(SmartMathis)
    Probe_ID = {pm.get_probe_ID(SmartMathis, i):[] for i in range(n_probe)}

    # Simulation parameters (start, end, time step) and time loop
    t = 0
    dt = 1
    t_final = 3600*4
    VentOn = 0
    while t < t_final:
        # lets set each controller's position
        for i, key in enumerate(PassiveCtrl_ID):
            pm.give_passive_ctrl(SmartMathis, PassiveCtrl_ID[key][-1], i)

        # Ask MATHIS to compute the current time step
        pm.solve_timestep(SmartMathis, t, dt)

        # lets fetch the new probes' value
        for i,key in enumerate(Probe_ID):
            Probe_ID[key].append(pm.get_probe_ctrl(SmartMathis,i))

        # Lets give the actuators new set points considering the probes values :
        # here is a simple action in which every 1/2hours 10 people are moving either inside or outside
        for key in PassiveCtrl_ID.keys():
            if key == 'CtrlNBPersonne':
                if t%3600<1800: PassiveCtrl_ID[key].append(10)  # 10 people in
                else: PassiveCtrl_ID[key].append(0)             # 0 people in
        # if CO2 probe give values above 800ppm, extracted airlow is at full capacity (define in the cas.data file)
        # a nd below 600ppm, its stopped
        if Probe_ID['SondeCO2'][-1]>800 and not VentOn:
            PassiveCtrl_ID['CtrlDebit'][-1] = 1 #it's a multiplier of the airflow define in the *.data file
            VentOn = 1
        elif Probe_ID['SondeCO2'][-1]<600 and VentOn:
            PassiveCtrl_ID['CtrlDebit'][-1] = 0
            VentOn = 0

        # updating time iteration
        t = t + dt
        # prompt consol report
        if t%1800==0: print('Time : ',t)
    #simulation is done, lets make a plot  of the control and probe
    MakeFinalPlot(PassiveCtrl_ID,Probe_ID)

def MakeFinalPlot(PassiveCtrl_ID,Probe_ID):
    plt.figure()
    plt.subplot(2,1,1)
    plt.plot(PassiveCtrl_ID['CtrlNBPersonne'])
    plt.ylabel(('CtrlNBPersonne'))
    plt.subplot(2, 1, 2)
    plt.plot(Probe_ID['SondeCO2'])
    plt.ylabel(('SondeCO2'))
    plt.xlabel(('Temps'))
    plt.show()

if __name__ == '__main__':
    main()


from setuptools import setup, find_packages

# setup(
#     name='pymathis',
#     version='0.1.0',
#     description='A library of interacting function to enable exchanges between MATHIS and python at each time steps',
#     url='https://scm.cstb.fr/cape/mathis/smartmathis',
#     author='Francois DEMOUGE & Xavier FAURE',
#     author_email='francois.demouge@cstb.fr & xavier.faure@cstb.fr',
#     license='GNU Lesser General Public License',
#     packages=['pymathis'],
#     install_requires=['matplotlib'],
#     packages=find_packages,
#     include_package_data=True,
#     classifiers=[
#         'Development Status :: 1 - Planning',
#         'Intended Audience :: Science/Research',
#         'License :: GNU Lesser General Public License ',
#         'Operating System :: OS Independent',
#         'Programming Language :: Python :: 3',
#     ],
#     python_requires=">=3.1"
# )

setup(
    name='pymathis',
    version='0.0.1',
    description='A library of interacting function to enable exchanges between MATHIS and python at each time steps',
    long_description='A library of interacting function to enable exchanges between MATHIS and python at each time steps',
    long_description_content_type='text/markdown',
    url='https://scm.cstb.fr/cape/mathis/smartmathis',
    author='Francois DEMOUGE, Xavier FAURE',
    author_email='francois.demouge@cstb.fr, xavier.faure@cstb.fr',
    license='GNU Lesser General Public License',
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
)



"""
This is an example of how to write an input *.data file for MATHIS.
In this specific example, the input file is named 'TestCase.data' which will be located
in a folder named 'TestCase' in the example folder of pymathis
One single function is used (CreateObj()) for each type of object of MATHIS
One can add as many attributes as available for each object
Undefined attribute will get default value.
For Object type, attributes and default value, please refer to MATHIS Documentation
"""

import pymathis.pymathis as pm
import os

FileName = 'TestCase'
Path2Case = os.path.join(os.getcwd(),'TestCase')

#File creation
Case = pm.CreateFile(Path2Case,FileName)
#SImulation parameters
Case = pm.CreateObj(Case,'MISC',TETA0=0, DTETA=1, TIMEUNIT='S', NSAVE=60, ISOTHERMAL=True, GUESS_POWER=True)
Case = pm.CreateObj(Case,'SPEC',ID='CO2',)
Case = pm.CreateObj(Case,'SPEC',ID='H2O')
#boundary conditions
Case = pm.CreateObj(Case,'EXT', TEXT=15, HR = 80)
#zones
Case = pm.CreateObj(Case,'LOC', ID='Room1', ALT=0, AREA=171, HEIGHT=2.525, TINI=20)
# branches
Case = pm.CreateObj(Case,'BRANCH', ID='ExtractionAir', BRANCHTYPE='DEBIT_CONSTANT', LOCIDS=['Room1','EXT'] ,Z1=2.5, Z2=2.5, QV0=2100, CTRLID='CtrlDebit')
Case = pm.CreateObj(Case,'BRANCH', ID='EntreeAir', BRANCHTYPE='ENTREE_FIXE', LOCIDS=['EXT','Room1'], Z1=2.3, Z2=2.3, QV0=1000, DPREF=20)
Case = pm.CreateObj(Case,'BRANCH', ID='Permea_bas', BRANCHTYPE='PERMEABILITE', LOCIDS=['EXT','Room1'],  Z1=0.5, Z2=0.5, QV0=7.5, DPREF=4)
#sources
Case = pm.CreateObj(Case,'HSRC', ID='SourceCO2', HSRCTYPE='STANDARD', LOCID='Room1', MFLUX=0.0000081, SPECIDS='CO2', YKS=1, CTRLID='CtrlNBPersonne')
#Operator
Case = pm.CreateObj(Case,'CTRL', ID='conversionCO2ppmv', CTRLTYPE='OPERATOR', FUNCTION='MAX', QUANTITIES='CONSTANT', CONSTANT=659090.91)
#probes
Case = pm.CreateObj(Case,'CTRL', ID='SondeCO2', CTRLTYPE='PROBE', FUNCTION='VALUE', QUANTITY='YK', SPECID='CO2', LOCID='Room1', CTRLID='conversionCO2ppmv')
Case = pm.CreateObj(Case,'CTRL', ID='SondeHR', CTRLTYPE='PROBE', FUNCTION='VALUE', QUANTITY='HR', LOCID='Room1')
#controler to play wth
Case = pm.CreateObj(Case,'CTRL', ID='CtrlDebit', CTRLTYPE='PASSIVE')
Case = pm.CreateObj(Case,'CTRL', ID='CtrlNBPersonne', CTRLTYPE='PASSIVE')

#closing file
Case.close()

